"""Climax configuration file."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import datetime

# externals
import numpy as np

# locals
from climax.core.constants import (PREDICTANDS, ERA5_P_VARIABLES,
                                   ERA5_S_VARIABLES)
from climax.core.models import SegNet, USegNet, StridedSegNet
from climax.core.loss import BernoulliGammaLoss, MSELoss, L1Loss

# -----------------------------------------------------------------------------
# ERA5 downscaling configuration ----------------------------------------------
# -----------------------------------------------------------------------------

# ERA5 predictor variables on pressure levels
ERA5_P_PREDICTORS = ['geopotential', 'temperature', 'u_component_of_wind',
                     'v_component_of_wind', 'specific_humidity']
assert all([var in ERA5_P_VARIABLES for var in ERA5_P_PREDICTORS])

# ERA5 predictor variables on single levels
# ERA5_S_PREDICTORS = ['mean_sea_level_pressure', 'orography', '2m_temperature']
ERA5_S_PREDICTORS = ['mean_sea_level_pressure']
assert all([var in ERA5_S_VARIABLES for var in ERA5_S_PREDICTORS])

# ERA5 predictor variables
ERA5_PREDICTORS = ERA5_P_PREDICTORS + ERA5_S_PREDICTORS

# ERA5 pressure levels
ERA5_PLEVELS = [500, 850]

# include day of year as predictor
DOY = False

# use digital elevation model instead of model orography
DEM = True
if DEM:
    # remove model orography when using DEM
    if 'orography' in ERA5_S_PREDICTORS:
        ERA5_S_PREDICTORS.remove('orography')

    # remove 2m temperature when using DEM
    # if '2m_temperature' in ERA5_S_PREDICTORS:
    #     ERA5_S_PREDICTORS.remove('2m_temperature')

# whether to use DEM slope and aspect as predictors
DEM_FEATURES = False

# stratify training/validation set for precipitation by number of wet days
STRATIFY = True

# size of the validation set w.r.t. the training set
# e.g., VALID_SIZE = 0.1 means: 90% of CALIB_PERIOD for training
#                               10% of CALIB_PERIOD for validation
VALID_SIZE = 0.1

# whether to train using cross-validation
CV = False

# -----------------------------------------------------------------------------
# Observations ----------------------------------------------------------------
# -----------------------------------------------------------------------------

# target variable: check if target variable is valid
PREDICTAND = 'tasmin'
assert PREDICTAND in PREDICTANDS

# threshold  defining the minimum amount of precipitation (mm) for a wet day
WET_DAY_THRESHOLD = 1

# -----------------------------------------------------------------------------
# Calibration and validation period -------------------------------------------
# -----------------------------------------------------------------------------

# calibration period: training and validation
CALIB_PERIOD = np.arange(
    datetime.datetime.strptime('1981-01-01', '%Y-%m-%d').date(),
    datetime.datetime.strptime('1991-01-01', '%Y-%m-%d').date())

# validation period: testing
VALID_PERIOD = np.arange(
    datetime.datetime.strptime('1991-01-01', '%Y-%m-%d').date(),
    datetime.datetime.strptime('2011-01-01', '%Y-%m-%d').date())

# -----------------------------------------------------------------------------
# Model training configuration ------------------------------------------------
# -----------------------------------------------------------------------------

# network architecture
NET = USegNet

# number of convolutional layers and corresponding filter sizes
FILTERS = [32, 64, 128, 256]

# loss function to optimize
# for temperature: L1Loss or MSELoss (NLL of normal distribution)
# for precipitation: BernoulliGammaLoss (NLL of Bernoulli-Gamma distribution)
LOSS = MSELoss()
# LOSS = BernoulliGammaLoss(min_amount=0)

# whether to randomly shuffle time steps or to conserve time series for model
# training
SHUFFLE = True

# whether to normalize the training data to [0, 1] (True) or to standardize to
# mean=0, std=1 (False)
NORM = True

# batch size: number of time steps processed by the net in each iteration
BATCH_SIZE = 16

# learning rate
LR = 0.001

# regularization rate
LAMBDA = 0

# network training configuration
TRAIN_CONFIG = {
    'checkpoint_state': {},
    'epochs': 250,
    'save': True,
    'save_loaders': False,
    'early_stop': True,
    'patience': 50,
    'multi_gpu': True,
    'classification': False,
    'clip_gradients': True
    }

# whether to overwrite existing models
OVERWRITE = False

# split reference period into NYEARS intervals
# this limits the amount of data that is loaded into memory during prediction
# NOTE: if NYEARS=None, load entire reference period into memory at once
NYEARS = 5
